package pep;
import java.io.*;
import java.util.*;

/**This class creates the randomized trial order for each participant.*/
public class DataSet{
 /**This method uses a source file to read what trials occur in a block,
  * randomizes them, and prints the complete sequence of trials to an
  * in[subject number].txt file.
  * @param source The name of the text file containing the task stimuli.
  * @param subj The participant number.
  * @param nBlocks The number of blocks of trials per participant.
  * @param isRepalce Replacement selection.
  * @param nSamples Number of trials.*/
 DataSet(String source,int subj,int nBlocks,boolean isReplace,int nSamples){
  try{
   //setup output writer and input reader
   String outFile = "in"+subj+".txt";
   FileOutputStream out = new FileOutputStream(outFile);
   PrintStream pIn = new PrintStream(out);
   BufferedReader br = new BufferedReader(new FileReader(source));

   String trials[] = new String[1000]; //array of trials
   int nTrials = 0; //trial counter
   Random r = new Random(); //random number generator

   /*read each line in the file passed to the buffered reader
     and save to the trials[] array*/
   for(boolean isFileEnd=false; !isFileEnd; nTrials++){
    trials[nTrials] = new String(br.readLine());
    //recording stops at "END_OF_FILE" marker
    if(trials[nTrials].equals("END_OF_FILE")){
     isFileEnd=true; //stops the looping
     nTrials--; //erases the END_OF_FILE entry
    }
   }

   //trial randomization
   if(isReplace){ //unblocked, with replacement
    for(int sample=0; sample<nSamples; sample++){
     int randNum = r.nextInt(nTrials);
     pIn.println(trials[randNum]);
    }
   }
   else{ //blocked, without replacement
    for(int block=0; block<nBlocks; block++){
     for(int trial=0; trial<nTrials; trial++){
      int randNum = r.nextInt(nTrials-trial);
      pIn.println(trials[randNum]);
      String temp = trials[randNum];
      trials[randNum] = trials[nTrials-trial-1];
      trials[nTrials-trial-1] = temp;
     }
    }
   }

   //leave an "END_OF_FILE" marker and close processes
   pIn.println("END_OF_FILE");
   pIn.close();
   out.close();
   br.close();
  }

  catch(Exception e){
   System.out.println("Error reading source");
  }
 }
}