/**Parallel Episodic Processing (PEP) Model
 * Version 1.2.0
 * James R. Schmidt, Universiteit Gent
 * Fonds Wetenschappelijk Onderzoek - Vlaanderen (FWO)*/

package pep;

import java.util.concurrent.Semaphore;

/**This is the Main class in which the model variables need to be
 * defined (in the VARIABLES section of the code).*/
public class Main{
 //----------VARIABLES------------------------------------------
 /**DataSet source*/
 private static String source = new String("hLowPC.txt");
 /**number of subjects*/                static int numSubjects = 1000;
 /**random with replacement?*/          static boolean isReplace = false;
 /**number of blocks per subject*/      private static int numBlocks = 1;
 /**number of trials (no replacement)*/ static int numSamples = 0;
 /**number of responses*/               static int numResponses = 6;
 /**number of targets*/                 static int numTargets = 6;
 /**target bias*/                       static double targBias = .9;
 /**target signal*/                     static double targSignal = .9;
 /**number of distracters per set*/     static int[] numDistracters = {6};
 /**distracter bias*/                   static double[] distBias = {.9};
 /**distracter signal*/                 static double[] distSignal = {.9};
 /**Set 1 distracter conflict*/         static boolean isConflict = true;
 /**number of conditons per variable*/  static int[] numPerVar = {2,2,2};
 /**number of cuncurrent threads*/      public static final int maxThreads = 4;
 //-------------------------------------------------------------
 public static final Semaphore freeThread = new Semaphore(maxThreads);
 public static RunTime runs[] = new RunTime[numSubjects];

 /**For each participant, the main method creates a DataSet, runs it
  * with RunTime (the guts of the model), and then generates condition
  * means with DataAnalysis.*/
 public static void main(String[] args) throws InterruptedException{
  for(int y=0; y<numSubjects; y++){
   freeThread.acquire();
   new DataSet(source,y+1,numBlocks,isReplace,numSamples);
   runs[y] = new RunTime(y);
   Thread t = new Thread(runs[y]);
   t.start();
   System.out.println(""+(y+1)+" of "+numSubjects+" subjects complete...");
  }

  //block until all threads are available (i.e., no active threads)
  freeThread.acquire(maxThreads);

  //analyse data and create concatenated raw dataset
  DataAnalysis.main(new String[0]);
  System.out.println("Data analysis complete...");
  Concat.main(new String[0]);
  System.out.println("Data files concatenated...");
 }
}