package pep;

import java.io.*;
import java.util.*;

/**This class is guts of the model, where each run is performed.*/
public class RunTime implements Runnable {
 /**number of targets*/          static int nTargets = Main.numTargets;
 /**number of distracter sets*/  static int nSets = Main.numDistracters.length;
 /**number of distracters*/      static int[] nDistract = Main.numDistracters;
 /**number of responses*/        static int nResponses = Main.numResponses;
 /**set 1 conflict*/             static boolean isConflict = Main.isConflict;
 /**number of variables*/        static int nVariables = Main.numPerVar.length;
 /**target signal strength*/     static double tSignal = Main.targSignal;
 /**distracter signal strength*/ static double[] dSignal = Main.distSignal;
 /**target processing bias*/     static double tBias = Main.targBias;
 /**distracter processing bias*/ static double[] dBias = Main.distBias;

 /**number of episodes*/
 static int nEpisodes = 1000; //should be greater than number of trials
 /**episodic memory*/
 Episode[] memory = new Episode[nEpisodes];
 /**episode counter*/
 int eNumber;
 /**target Input nodes*/
 Input[] targets = new Input[nTargets];
 /**distracter Input nodes*/
 Input[][] distracters = new Input[nSets][];
 /**Identity nodes*/
 Identity[] identities = new Identity[nTargets];
 /**Response nodes*/
 Response[] responses = new Response[nResponses];

 /**end of trial marker*/  boolean isTrialOver;
 /**subject number*/       int subj;
 /**condition number*/     int[] condition = new int[nVariables];
 /**target number*/        int target;
 /**distrater number*/     int[] distracter = new int[nSets];
 /**cycle number*/         int cycle;
 /**correct response*/     int correctResp;
 /**actual response*/      int response;
 /**accuracy*/             int accuracy;
 /**stimulus on screen?*/  boolean isStimuli = true;

 /**Sets up the thread.
  * @param subj The subject number.*/
 @SuppressWarnings("empty-statement")
 public RunTime(int subject){
  subj=subject;
 }

 /**Defines the nodes of the model defined and runs the trials.*/
 public void run(){
  //sets up the nodes for the distracter sets
  for(int set=0; set<nSets; set++){
   distracters[set] = new Input[nDistract[set]];
   for(int k=0; k<nDistract[set]; k++){
    distracters[set][k] = new Input(subj);
    distracters[set][k].ignored(nSets);
   }
  }
  //sets up the rest of the nodes
  for(int i=0; i<nEpisodes; i++) memory[i] = new Episode(subj);
  for(int i=0; i<nTargets; i++){
   targets[i] = new Input(subj);
   targets[i].targeted();
   identities[i] = new Identity(subj);
  }
  for(int i=0; i<nResponses; i++) responses[i] = new Response(subj);
  //set episode and connection counters to 0
  eNumber = 0; Node.nConnections[subj] = eNumber;

  //---Open I/O Streams---
  StringTokenizer tokenizer = null;
  BufferedReader in = null;
  FileOutputStream out = null;
  PrintStream p = null;
  try{
   String inFile = "in"+(subj+1)+".txt";
   in = new BufferedReader(new FileReader(inFile));
   String outFile = "out"+(subj+1)+".txt";
   out = new FileOutputStream(outFile);
   p = new PrintStream(out);
   tokenizer = new StringTokenizer(in.readLine()); //<--Trial 1
  }
  catch(Exception e){
   System.out.println("Error setting up input and output streams.");
  }

  //---Trial Loop---
  while(in!=null){
   //split up trial info into variables
   correctResp = Integer.parseInt(tokenizer.nextToken());
   target = Integer.parseInt(tokenizer.nextToken());
   for(int ns=0; ns<nSets; ns++)
    distracter[ns] = Integer.parseInt(tokenizer.nextToken());
   for(int nc=0; nc<nVariables; nc++)
    condition[nc] = Integer.parseInt(tokenizer.nextToken());
   response = 999; accuracy = 999;

   //---Define Inputs---
   //noise activation is added to each input node
   for(int i=0; i<nTargets; i++) targets[i].param(tBias,0);
   for(int j=0; j<nSets; j++)
    for(int i=0; i<nDistract[j]; i++) distracters[j][i].param(dBias[j],0);

   //the presented nodes are activated with noise plus a signal
   if(nTargets>0) targets[target].param(tBias,tSignal);
   for(int j=0; j<nSets; j++)
    if(nDistract[j]>0)
     distracters[j][distracter[j]].param(dBias[j],dSignal[j]);

   //---Run Cycles---
   isTrialOver = false; isStimuli = true;
   //run a cycle and monitor for colour and response identification
   for(cycle=1; isTrialOver==false; cycle++){
    go();
    if(monitorTarg()) if(monitorResp()) isTrialOver=true;
    if(cycle>1999) isTrialOver=true;
   }

   //---Post-Trial Coding---
   //calculate accuracy
   if(correctResp==response) accuracy = 1;
   else{
    accuracy = 0;
    memory[eNumber].tagError();
   }
   //write trial info to file
   p.print(subj+"\t"+response+"\t"+target+"\t");
   for(int k=0; k<nSets; k++) p.print(distracter[k]+"\t");
   for(int m=0; m<nVariables; m++) p.print(condition[m]+"\t");
   p.println(correctResp+"\t"+cycle+"\t"+accuracy);
   //record response time into memory
   memory[eNumber].setRT(cycle);

   //---Run ITI---
   isStimuli = false;
   //null input cycles (inter-trial interval)
   for(int i=0; i<nTargets; i++) targets[i].param(0,0);
   for(int j=0; j<nSets; j++)
    for(int i=0; i<nDistract[j]; i++) distracters[j][i].param(0,0);
   //300 cycle ITI
   for(int i=0; i<300; i++){
    go();
   }

   //increase episode and connection counters
   eNumber++; Node.nConnections[subj] = eNumber;
   if(eNumber>=nEpisodes) System.out.println("Error: Not enough Episodes.");

   //---Read Next Trial---
   try{
    String tempTrial = new String(in.readLine());
    if(tempTrial.equals("END_OF_FILE")) in = null;
    else tokenizer = new StringTokenizer(tempTrial);
   }
   catch(Exception e2){
    System.out.println("No END_OF_FILE marker");
    in = null;
   }
  }

  //---Exit Simulation---
  p.println("END_OF_FILE"); //tag end of file
  //close input and output streams
  try{
   p.close(); out.close();
  }
  catch(Exception e3){
   System.out.println("Error closing input and output streams.");
  }

  //release thread to freeThread semaphore to allow a new thread
  Main.freeThread.release();
 }


 //-------------------------------------------------------------------
 //------------------------------Methods------------------------------
 //-------------------------------------------------------------------

 /**Performs all the calculations for an individual cycle.*/
 private void go(){
  //---INPUTS---
  for(int j=0; j<nSets; j++){
   boolean isConf = isConflict;
   if(j>0) isConf=false; //only the first node set conflicts
   for(int i=0; i<nDistract[j]; i++) distracters[j][i].activate(i,isConf);
  }
  for(int i=0; i<nTargets; i++) targets[i].activate(i,isConflict);
  //---EPISODES---
  for(int i=0; i<eNumber; i++) memory[i].activate();
  memory[eNumber].focus();
  if(isStimuli) memory[eNumber].connect();
  //---IDENTITIES---
  for(int i=0; i<nTargets; i++) identities[i].activate(i);
  Identity.clear(subj);
  //---RESPONSES---
  for(int i=0; i<nResponses; i++) responses[i].activate();
  Response.clear(subj);
 }

 /**Checks colour colour identification.
  *@return If target is present.*/
 private boolean monitorTarg(){
  boolean found = false;
  for(int i=0; i<nTargets; i++)
   if(targets[i].activation>targets[i].threshold) found = true;
  return found;
 }

 /**Checks for temporal expectancy and generates a temporal bias.
  *@return Temporal expectancy bias.*/
 private double monitorTemp(){
  double expect = 0;
  //Limit search on initial trials (to avoid runtime errors)
  int iLimit = 40; if(eNumber<40) iLimit=eNumber;
  //Search for temporal expectancies
  for(int i=1; i<=iLimit; i++)
   expect += memory[eNumber-i].getRT(cycle,(Math.pow((40-i),2)*.00002));
  expect -= .01; if(expect<0) expect=0; //small threshold
  if(expect>.15) expect=.15; //max expectancy drop
  return expect;
 }

 /**Checks for response identification and ends the trial if necessary.
  *@return If the response has been found.*/
 private boolean monitorResp(){
  boolean found = false;
  for(int i=0; i<nResponses; i++){
   if(responses[i].activation>(responses[i].threshold-monitorTemp())){
    found = true;
    //if two responses cross threshold at same time, response = 777
    if(response != 999) response = 777;
    else response = i;
   }
  }
  return found;
 }
}